unit fMain;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ImgList,
  ActnList,
  ActnMan,
  ActnCtrls,
  ActnMenus,
  XPMan,
  XPStyleActnCtrls,
  ToolWin,
  ComCtrls,
  ExtCtrls,
  StdCtrls,
  Buttons,
  IniFiles,
  uGlobalVars,
  iCPTerminology,
  oMRUList,
  Menus,
  StdActns;

type
  TfrmMain = class(TForm)
    CoolBar1: TCoolBar;
    acMgr: TActionManager;
    acOpen: TAction;
    acSave: TAction;
    acSaveAs: TAction;
    acClose: TAction;
    acExit: TAction;
    ImageList1: TImageList;
    ActionMainMenuBar1: TActionMainMenuBar;
    StatusBar1: TStatusBar;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    lblMappingTable: TLabel;
    lblEntries: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    bbtnNewKey: TBitBtn;
    bbtnDeleteKey: TBitBtn;
    acNewKey: TAction;
    acDeleteKey: TAction;
    chkbxDoNotProcessThisKey: TCheckBox;
    cbxObservation: TComboBox;
    cbxUnit: TComboBox;
    cbxMethod: TComboBox;
    cbxLocation: TComboBox;
    cbxQuality: TComboBox;
    cbxProduct: TComboBox;
    lblMappingLogic: TLabel;
    Shape3: TShape;
    dlgOpen: TOpenDialog;
    edtName: TEdit;
    lblName: TLabel;
    edtSourceID: TEdit;
    edtSourceApplication: TEdit;
    edtSourceVersion: TEdit;
    edtHL7SendingApplication: TEdit;
    lblSendingApplication: TLabel;
    lblSourceID: TLabel;
    lblSourceApplication: TLabel;
    lblSourceVersion: TLabel;
    lblComment: TLabel;
    cbxActive: TCheckBox;
    memComment: TMemo;
    chkbxSourceTrusted: TCheckBox;
    acNewTable: TAction;
    acLoadTable: TAction;
    SaveDialog1: TSaveDialog;
    bbtnRename: TBitBtn;
    acRename: TAction;
    acDoNotProcessCheck: TAction;
    acSelectObservation: TAction;
    lvKeys: TListView;
    lblObservation: TLabel;
    lblUnit: TLabel;
    lblMethod: TLabel;
    lblLocation: TLabel;
    lblQuality: TLabel;
    lblProduct: TLabel;
    acSelectMethod: TAction;
    acSelectUnit: TAction;
    acSelectLocation: TAction;
    acSelectQuality: TAction;
    acSelectProduct: TAction;
    acValidate: TAction;
    ToolButton3: TToolButton;
    acInstall: TAction;
    pmnMRU: TPopupMenu;
    acEditUndo: TEditUndo;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    acBatchUpdate: TAction;
    acLoadEntries: TAction;
    procedure FormCreate(Sender: TObject);
    procedure acExitExecute(Sender: TObject);
    procedure acNewTableExecute(Sender: TObject);
    procedure acMgrUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acLoadTableExecute(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acSaveAsExecute(Sender: TObject);
    procedure acNewKeyExecute(Sender: TObject);
    procedure acDoNotProcessCheckExecute(Sender: TObject);
    procedure acSelectObservationExecute(Sender: TObject);
    procedure lvKeysSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure acSelectMethodExecute(Sender: TObject);
    procedure acSelectUnitExecute(Sender: TObject);
    procedure acSelectLocationExecute(Sender: TObject);
    procedure acSelectQualityExecute(Sender: TObject);
    procedure acSelectProductExecute(Sender: TObject);
    procedure ClearComboBox(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure acRenameExecute(Sender: TObject);
    procedure acDeleteKeyExecute(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
    procedure lvKeysColumnClick(Sender: TObject; Column: TListColumn);
    procedure acValidateExecute(Sender: TObject);
    procedure acInstallExecute(Sender: TObject);
    procedure lvKeysGetImageIndex(Sender: TObject; Item: TListItem);
    procedure lvKeysCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure edtNameChange(Sender: TObject);
    procedure edtHL7SendingApplicationChange(Sender: TObject);
    procedure edtSourceIDChange(Sender: TObject);
    procedure edtSourceApplicationChange(Sender: TObject);
    procedure edtSourceVersionChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cbxActiveClick(Sender: TObject);
    procedure chkbxSourceTrustedClick(Sender: TObject);
    procedure acBatchUpdateExecute(Sender: TObject);
    procedure acLoadEntriesExecute(Sender: TObject);
  private
    fCurrentEntry: ITermMappingTableEntry;
    fLoading: boolean;
    fDoNotProcessTerm: ITerm;
    fSortColumn: integer;
    fReverseSort: boolean;
    fInstallable: boolean;
    fTerminologyFile: string;
    fTerminologyPath: string;
    fSettingsFile: TMemIniFile;
    fMRUList: TMRUList;
    function LoadTerminology: boolean;
    procedure LoadMappingTable(const aFilename: string);
    procedure MissingMappingTable(const aFilename: string);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  fInstaller,
  fGlobalUpdate;

{$R *.dfm}

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  fSettingsFile.UpdateFile;
  acClose.Execute;
  CanClose := True;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  fSettingsFile := TMemIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Font := Screen.IconFont;
  lblMappingTable.Font.Style := [fsBold];
  lblEntries.Font.Style := [fsBold];
  MyTerminology := NewTerminologyController;
  MyMappingTable := NewMappingTable(MyTerminology);
  Show;
  Application.ProcessMessages;
  fInstallable := FindCmdLineSwitch('useinstaller', True);
  fMRUList := TMRUList.Create(pmnMRU, fSettingsFile);
  fMRUList.OnMenuClick := LoadMappingTable;
  fMRUList.OnFileNotExist := MissingMappingTable;

  try
    if LoadTerminology then
      begin
        {Ask the user to verify that this is what they want to use}
        with MyTerminology do
          if MessageDlg('Terminology Loaded' + #13 + #13 +
            'Filename: ' + ExpandFileName(fTerminologyFile) + #13 + #13 +
            'Description: ' + getDescription + #13 +
            'Created: ' + DateTimeToStr(getDateTimeCreated) + #13 +
            'Last Modified: ' + DateTimeToStr(getDateTimeModified) + #13 + #13 +
            'Is this the correct terminology set?',
            mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
            raise Exception.Create('User requested shutdown.');

        fDoNotProcessTerm := MyTerminology.getTerm(DO_NOT_PROCESS_ID);

        if not Assigned(fDoNotProcessTerm) then
          raise Exception.Create('No term assigned for DO NOT PROCESS');
      end
    else
      raise Exception.Create('Error loading terminology file.');
  except
    on E: Exception do
      begin
        MessageDlg(
          'Error encountered starting application.' + #13 + #13 +
          E.Message, mtError, [mbOk], 0);
        Application.Terminate;
      end;
  end;
end;

procedure TfrmMain.LoadMappingTable(const aFilename: string);
begin
  try
    if MyMappingTable.getIsOpen then
      try
        acClose.Execute;
      except
        on E: Exception do
          MessageDlg(E.Message, mtError, [mbOk], 0);
      end;

    MyMappingTable.Load(aFileName);
    acLoadTable.Execute;
    MyMappingTable.setChangeMade(False);
    fSettingsFile.WriteString('Mapping_Table', 'Path', ExtractFilePath(aFileName));
    fSettingsFile.WriteString('Mapping_Table', 'File', ExtractFileName(aFileName));
    fMRUList.AddFile(aFileName);
  except
    fMRUList.RemoveFile(aFileName);
  end;
end;

function TfrmMain.LoadTerminology: boolean;
begin
  {Ok, let's get some terminology loaded up first}
  with fSettingsFile do
    try
      fTerminologyPath := ReadString('Terminology', 'Path', '.\');
      fTerminologyFile := ReadString('Terminology', 'File', 'CPTerminology.xml');
      dlgOpen.Title := 'Select CP Terminology';
      dlgOpen.InitialDir := fTerminologyPath;
      dlgOpen.FileName := fTerminologyFile;
      dlgOpen.Filter := 'Mapping Table Files|*.xml';
      dlgOpen.DefaultExt := '.xml';
      if dlgOpen.Execute then
        try
          fTerminologyFile := dlgOpen.FileName;
          MyTerminology.Load(fTerminologyFile);
          MyTerminology.LoadList(cbxObservation.Items, ttObservation);
          WriteString('Terminology', 'Path', ExtractFilePath(fTerminologyFile));
          WriteString('Terminology', 'File', ExtractFileName(fTerminologyFile));
        except
          raise Exception.CreateFmt('Unable to open %s', [fTerminologyFile]);
        end
      else
        raise Exception.Create('Must have a valid terminology file to continue.');
      Result := True;
    except
      on E: Exception do
        begin
          MessageDlg(E.Message, mtError, [mbOk], 0);
          Result := False;
        end;
    end;
end;

procedure TfrmMain.lvKeysColumnClick(Sender: TObject; Column: TListColumn);
var
  i: integer;
begin
  if fSortColumn = Column.Index then {Clicking same column - reverse the sort}
    fReverseSort := not fReverseSort
  else
    fReverseSort := False;

  fSortColumn := Column.Index;
  for i := 0 to lvKeys.Columns.Count - 1 do
    lvKeys.Columns[i].ImageIndex := -1;
  if fReverseSort then
    lvKeys.Columns[fSortColumn].ImageIndex := 9
  else
    lvKeys.Columns[fSortColumn].ImageIndex := 10;
  lvKeys.AlphaSort;
end;

procedure TfrmMain.lvKeysCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  case fSortColumn of
    0: {Sorts by vendor key}
      begin
        Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);
      end;
    1: {Sorts by observation description}
      begin
        Compare := AnsiCompareStr(Item1.SubItems[0], Item2.SubItems[0]);
      end;
  end;
  if Compare = 0 then
    Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);

  if fReverseSort then
    Compare := Compare * -1;
end;

procedure TfrmMain.lvKeysGetImageIndex(Sender: TObject; Item: TListItem);
var
  aEntry: ITermMappingTableEntry;
begin
  if TObject(Item.Data).GetInterface(ITermMappingTableEntry, aEntry) then
    begin
      if aEntry.getDoNotProcess then
        Item.ImageIndex := 7
      else
        Item.ImageIndex := 8;
    end;
end;

procedure TfrmMain.lvKeysSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  fLoading := True;
  if Selected then
    try
      TObject(Item.Data).GetInterface(ITermMappingTableEntry, fCurrentEntry);
      chkbxDoNotProcessThisKey.Checked := fCurrentEntry.getDoNotProcess;
      if Assigned(fCurrentEntry.getMappedTerm(ttObservation)) then
        begin
          cbxObservation.ItemIndex := fCurrentEntry.getMappedTerm(ttObservation).getListIndex(cbxObservation.Items);
          fCurrentEntry.getMappedTerm(ttObservation).LoadComponentTerms(cbxUnit.Items, ttUnit);
          fCurrentEntry.getMappedTerm(ttObservation).LoadComponentTerms(cbxMethod.Items, ttMethod);
          fCurrentEntry.getMappedTerm(ttObservation).LoadComponentTerms(cbxLocation.Items, ttLocation);
          fCurrentEntry.getMappedTerm(ttObservation).LoadComponentTerms(cbxQuality.Items, ttQuality);
          fCurrentEntry.getMappedTerm(ttObservation).LoadComponentTerms(cbxProduct.Items, ttProduct);
        end
      else
        begin
          MyTerminology.LoadList(cbxUnit.Items, ttUnit);
          MyTerminology.LoadList(cbxMethod.Items, ttMethod);
          MyTerminology.LoadList(cbxLocation.Items, ttLocation);
          MyTerminology.LoadList(cbxQuality.Items, ttQuality);
          MyTerminology.LoadList(cbxProduct.Items, ttProduct);
        end;

      if Assigned(fCurrentEntry.getMappedTerm(ttUnit)) then
        cbxUnit.ItemIndex := fCurrentEntry.getMappedTerm(ttUnit).getListIndex(cbxUnit.Items);

      if Assigned(fCurrentEntry.getMappedTerm(ttMethod)) then
        cbxMethod.ItemIndex := fCurrentEntry.getMappedTerm(ttMethod).getListIndex(cbxMethod.Items);

      if Assigned(fCurrentEntry.getMappedTerm(ttLocation)) then
        cbxLocation.ItemIndex := fCurrentEntry.getMappedTerm(ttLocation).getListIndex(cbxLocation.Items);

      if Assigned(fCurrentEntry.getMappedTerm(ttQuality)) then
        cbxQuality.ItemIndex := fCurrentEntry.getMappedTerm(ttQuality).getListIndex(cbxQuality.Items);

      if Assigned(fCurrentEntry.getMappedTerm(ttProduct)) then
        cbxProduct.ItemIndex := fCurrentEntry.getMappedTerm(ttProduct).getListIndex(cbxProduct.Items);
    except
      on E: Exception do
        MessageDlg('Error accessing mapping entry.' + #13 + E.Message, mtError, [mbOk], 0);
    end
  else
    begin
      if Assigned(fCurrentEntry) then
        fCurrentEntry.Save;
      fCurrentEntry := nil;
      cbxObservation.ItemIndex := -1;
      cbxUnit.ItemIndex := -1;
      cbxMethod.ItemIndex := -1;
      cbxLocation.ItemIndex := -1;
      cbxQuality.ItemIndex := -1;
      cbxProduct.ItemIndex := -1;
    end;
  fLoading := False;
end;

procedure TfrmMain.MissingMappingTable(const aFilename: string);
begin
  MessageDlg('File ' + aFileName + ' does not exist', mtError, [mbOk], 0);
  fMRUList.RemoveFile(aFileName);
end;

procedure TfrmMain.acCloseExecute(Sender: TObject);
begin
  if MyMappingTable.getIsOpen then
    if MyMappingTable.getChangeMade then
      if MessageDlg(Format('Save Changes To %s Before Closing?', [ExtractFileName(MyMappingTable.getFileName)]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        try
          MyMappingTable.Save;
        except
          on E: Exception do
            MessageDlg('Error saving Mapping Table:' + #13 + '   ' + E.Message, mtError, [mbOk], 0);
        end;
  MyMappingTable.Close;
  edtName.Text := '';
  edtSourceID.Text := '';
  edtSourceApplication.Text := '';
  edtSourceVersion.Text := '';
  edtHL7SendingApplication.Text := '';
  cbxActive.Checked := False;
  chkbxSourceTrusted.Checked := False;
  memComment.Text := '';
  lvKeys.Items.Clear;
end;

procedure TfrmMain.acDeleteKeyExecute(Sender: TObject);
begin
  if MessageDlg('Are you sure you want to delete this item?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
      MyMappingTable.DeleteEntry(fCurrentEntry);
      lvKeys.DeleteSelected;
    end;
end;

procedure TfrmMain.acDoNotProcessCheckExecute(Sender: TObject);
var
  aTermType: TTermType;
begin
  if not fLoading then
    begin
      fCurrentEntry.setDoNotProcess(chkbxDoNotProcessThisKey.Checked);
      for aTermType := Low(TTermType) to High(TTermType) do
        fCurrentEntry.setMappedTerm(aTermType, nil);
      {
      fCurrentEntry.setObservation(nil);
      fCurrentEntry.setUnit(nil);
      fCurrentEntry.setMethod(nil);
      fCurrentEntry.setLocation(nil);
      fCurrentEntry.setQuality(nil);
      fCurrentEntry.setProduct(nil);
      }
      cbxObservation.ItemIndex := -1; { Never unload terms :) }

      MyTerminology.LoadList(cbxUnit.Items, ttUnit);
      cbxUnit.ItemIndex := -1;

      MyTerminology.LoadList(cbxMethod.Items, ttMethod);
      cbxMethod.ItemIndex := -1;

      MyTerminology.LoadList(cbxLocation.Items, ttLocation);
      cbxLocation.ItemIndex := -1;

      MyTerminology.LoadList(cbxQuality.Items, ttQuality);
      cbxQuality.ItemIndex := -1;

      MyTerminology.LoadList(cbxProduct.Items, ttProduct);
      cbxProduct.ItemIndex := -1;

      if fCurrentEntry.getDoNotProcess then
        begin
          fCurrentEntry.setMappedTerm(ttObservation, fDoNotProcessTerm);
          cbxObservation.ItemIndex := fDoNotProcessTerm.getListIndex(cbxObservation.Items);
        end;

      lvKeys.Selected.SubItems.Clear;
      lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
      lvKeys.Invalidate;
    end;
end;

procedure TfrmMain.acSelectObservationExecute(Sender: TObject);
var
  aTerm: ITerm;
  aTermType: TTermType;
begin
  if not fLoading then
    begin
      {Whenever the observation changes in the UI we clear the others}
      for aTermType := Low(TTermType) to High(TTermType) do
        fCurrentEntry.setMappedTerm(aTermType, nil);
      {
      fCurrentEntry.setObservation(nil);
      fCurrentEntry.setUnit(nil);
      fCurrentEntry.setMethod(nil);
      fCurrentEntry.setLocation(nil);
      fCurrentEntry.setQuality(nil);
      fCurrentEntry.setProduct(nil);
      }
      if cbxObservation.ItemIndex > -1 then
        begin {Load obs specific items}
          if cbxObservation.Items.Objects[cbxObservation.ItemIndex].GetInterface(ITerm, aTerm) then
            begin
              fCurrentEntry.setMappedTerm(ttObservation, aTerm);

              cbxUnit.ItemIndex := -1;
              aTerm.LoadComponentTerms(cbxUnit.Items, ttUnit);

              cbxMethod.ItemIndex := -1;
              aTerm.LoadComponentTerms(cbxMethod.Items, ttMethod);

              cbxLocation.ItemIndex := -1;
              aTerm.LoadComponentTerms(cbxLocation.Items, ttLocation);

              cbxQuality.ItemIndex := -1;
              aTerm.LoadComponentTerms(cbxQuality.Items, ttQuality);

              cbxProduct.ItemIndex := -1;
              aTerm.LoadComponentTerms(cbxProduct.Items, ttProduct);

              lvKeys.Selected.SubItems.Clear;
              lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
            end
        end
      else
        begin {Load all items}
          MyTerminology.LoadList(cbxUnit.Items, ttUnit);
          MyTerminology.LoadList(cbxMethod.Items, ttMethod);
          MyTerminology.LoadList(cbxLocation.Items, ttLocation);
          MyTerminology.LoadList(cbxQuality.Items, ttQuality);
          MyTerminology.LoadList(cbxProduct.Items, ttProduct);

          lvKeys.Selected.SubItems.Clear;
          lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
        end;
    end;
end;

procedure TfrmMain.acSelectProductExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if not fLoading then
    begin
      fCurrentEntry.setMappedTerm(ttProduct, nil);
      if cbxProduct.ItemIndex > -1 then
        if cbxProduct.Items.Objects[cbxProduct.ItemIndex].GetInterface(ITerm, aTerm) then
          begin
            fCurrentEntry.setMappedTerm(ttProduct, aTerm);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
          end;
    end;
end;

procedure TfrmMain.acSelectQualityExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if not fLoading then
    begin
      fCurrentEntry.setMappedTerm(ttQuality, nil);
      if cbxQuality.ItemIndex > -1 then
        if cbxQuality.Items.Objects[cbxQuality.ItemIndex].GetInterface(ITerm, aTerm) then
          begin
            fCurrentEntry.setMappedTerm(ttQuality, aTerm);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
          end;
    end;
end;

procedure TfrmMain.acSelectUnitExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if not fLoading then
    begin
      fCurrentEntry.setMappedTerm(ttUnit, nil);
      if cbxUnit.ItemIndex > -1 then
        if cbxUnit.Items.Objects[cbxUnit.ItemIndex].GetInterface(ITerm, aTerm) then
          begin
            fCurrentEntry.setMappedTerm(ttUnit, aTerm);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
          end;
    end;
end;

procedure TfrmMain.acValidateExecute(Sender: TObject);
var
  aList: TStringList;
  aEntry: ITermMappingTableEntry;
  i: integer;
  aMsg: string;
begin
  aList := TStringList.Create;
  try
    with MyMappingTable.NewEnumerator do
      while MoveNext do
        if not GetCurrentEntry.getDoNotProcess then {Only worried about the ones we actually process}
          begin
            i := aList.IndexOf(GetCurrentEntry.getMappingIDs);
            if i > -1 then
              begin
                aList.Objects[i].GetInterface(ITermMappingTableEntry, aEntry);
                aMsg := Format('Possible mapping issue.' + #13 + #13 +
                  'Vendor Keys: ''%s'' and ''%s'' are mapped identically.',
                  [getCurrentEntry.getVendorKey, aEntry.getVendorKey]);
                MessageDlg(aMsg, mtConfirmation, [mbOk], 0);
              end
            else
              begin
                i := GetCurrentEntry.AddToList(aList);
                aList[i] := GetCurrentEntry.getMappingIDs;
              end;
          end;
  finally
    FreeAndNil(aList);
  end;
end;

procedure TfrmMain.ClearComboBox(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  aNotify: TNotifyEvent;
begin
  if Sender is TComboBox then
    if Key = VK_ESCAPE then
      begin
        (Sender as TComboBox).ItemIndex := -1;
        aNotify := (Sender as TComboBox).OnChange;
        if Assigned(aNotify) then
          aNotify(Sender);
      end;
end;

procedure TfrmMain.cbxActiveClick(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setActive(cbxActive.Checked);
end;

procedure TfrmMain.chkbxSourceTrustedClick(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setSourceTrusted(chkbxSourceTrusted.Checked);
end;

procedure TfrmMain.edtHL7SendingApplicationChange(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setHL7SendingApplication(edtHL7SendingApplication.Text);
end;

procedure TfrmMain.edtNameChange(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setName(edtName.Text);
end;

procedure TfrmMain.edtSourceApplicationChange(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setSourceApplication(edtSourceApplication.Text);
end;

procedure TfrmMain.edtSourceIDChange(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setSourceID(edtSourceID.Text);
end;

procedure TfrmMain.edtSourceVersionChange(Sender: TObject);
begin
  if not fLoading then
    MyMappingTable.setSourceVersion(edtSourceVersion.Text);
end;

procedure TfrmMain.acSelectLocationExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if not fLoading then
    begin
      fCurrentEntry.setMappedTerm(ttLocation, nil);
      if cbxLocation.ItemIndex > -1 then
        if cbxLocation.Items.Objects[cbxLocation.ItemIndex].GetInterface(ITerm, aTerm) then
          begin
            fCurrentEntry.setMappedTerm(ttLocation, aTerm);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
          end;
    end;
end;

procedure TfrmMain.acSelectMethodExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if not fLoading then
    begin
      fCurrentEntry.setMappedTerm(ttMethod, nil);
      if cbxMethod.ItemIndex > -1 then
        if cbxMethod.Items.Objects[cbxMethod.ItemIndex].GetInterface(ITerm, aTerm) then
          begin
            fCurrentEntry.setMappedTerm(ttMethod, aTerm);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(fCurrentEntry.getMappingDisplay);
          end;
    end;
end;

procedure TfrmMain.acExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.acBatchUpdateExecute(Sender: TObject);
begin
  PerformGlobalUpdate;
  acLoadEntries.Execute;
end;

procedure TfrmMain.acInstallExecute(Sender: TObject);
begin
  InstallMappingTable(MyMappingTable);
end;

procedure TfrmMain.acLoadEntriesExecute(Sender: TObject);
begin
  try
    lvKeys.Items.BeginUpdate;
    lvKeys.Items.Clear;
    with MyMappingTable.NewEnumerator do
      while MoveNext do
        GetCurrentEntry.AddToList(lvKeys, -1).SubItems.Add(GetCurrentEntry.getMappingDisplay);
    fSortColumn := -1;
    lvKeysColumnClick(Sender, lvKeys.Column[0]);
  finally
    lvKeys.Items.EndUpdate;
  end;
end;

procedure TfrmMain.acLoadTableExecute(Sender: TObject);
begin
  fLoading := True;
  try
    edtName.Text := MyMappingTable.getName;
    edtSourceID.Text := MyMappingTable.getSourceID;
    edtSourceApplication.Text := MyMappingTable.GetSourceApplication;
    edtSourceVersion.Text := MyMappingTable.GetSourceVersion;
    edtHL7SendingApplication.Text := MyMappingTable.getHL7SendingApplication;

    cbxActive.Checked := MyMappingTable.getActive;
    chkbxSourceTrusted.Checked := MyMappingTable.getSourceTrusted;

    memComment.Text := MyMappingTable.getComment;
    acLoadEntries.Execute;
  finally
    fLoading := False;
  end;
end;

procedure TfrmMain.acNewKeyExecute(Sender: TObject);
var
  aNewKey: string;
  aNewEntry: ITermMappingTableEntry;
  aNewKeyItem: TListItem;
begin
  aNewKey := '';
  if InputQuery('New Vendor Key', 'Key Value:', aNewKey) then
    if aNewKey <> '' then
      begin
        aNewKeyItem := nil;
        with lvKeys.Items.GetEnumerator do
          while MoveNext do
            if GetCurrent.Caption = aNewKey then
              aNewKeyItem := GetCurrent;

        if Assigned(aNewKeyItem) then
          begin
            if MessageDlg('Key already exists. Select it for editing?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
              lvKeys.Selected := aNewKeyItem;
          end
        else
          begin
            aNewEntry := MyMappingTable.NewEntry(aNewKey); //lbxKeys.Items.Add(aNewKey); {Obviously need some work here}
            lvKeys.Selected := aNewEntry.AddToList(lvKeys);
            lvKeys.Selected.SubItems.Clear;
            lvKeys.Selected.SubItems.Add(aNewEntry.getMappingDisplay);
            lvKeys.Selected.ImageIndex := 2;
            lvKeys.AlphaSort;
            lvKeys.Selected.MakeVisible(False);
            lvKeys.SetFocus;
          end
      end
    else
      MessageDlg('Blank key value not allowed.', mtError, [mbYes, mbNo], 0);
end;

procedure TfrmMain.acRenameExecute(Sender: TObject);
var
  aNewValue: string;
  aDuplicate: Boolean;
begin
  if InputQuery('Rename Vendor Key', 'New Key Value:', aNewValue) then
    if aNewValue <> '' then
      begin
        aDuplicate := False;
        with lvKeys.Items.GetEnumerator do
          while MoveNext do
            if (GetCurrent.Caption = aNewValue) and (GetCurrent.Index <> lvKeys.Selected.Index) then
              aDuplicate := True;
        if aDuplicate then
          MessageDlg('That vendor key already exists for this mapping table.', mtError, [mbOk], 0)
        else
          begin
            fCurrentEntry.setVendorKey(aNewValue);
            lvKeys.Selected.Caption := aNewValue;
          end;
      end
    else
      MessageDlg('Blank Keys not allowed.', mtError, [mbOk], 0);
end;

procedure TfrmMain.acNewTableExecute(Sender: TObject);
begin
  if MyMappingTable.getIsOpen then
    ShowMessage('You need to check the status of the old file first');

  with TOpenDialog.Create(Self) do
    try
      Filter := 'Mapping Table Files|*.xml';
      DefaultExt := '.xml';
      Title := 'Open Mapping Table File';
      Options := [ofOverwritePrompt, ofHideReadOnly, ofPathMustExist, ofEnableSizing];
      if Execute then
        begin
          MyMappingTable.Load(FileName);
          MyMappingTable.Save;
          acLoadTable.Execute;
        end;
    finally
      Free;
    end;
end;

procedure TfrmMain.acOpenExecute(Sender: TObject);
var
  aPath: string;
  aFile: string;
begin
  with fSettingsFile do
    try
      aPath := ReadString('Mapping_Table', 'Path', '.\');
      aFile := ReadString('Mapping_Table', 'File', '');

      with dlgOpen do
        try
          Filter := 'Mapping Table Files|*.xml';
          DefaultExt := '.xml';
          InitialDir := aPath;
          FileName := aFile;
          Title := 'Open Mapping Table File';
          Options := [ofHideReadOnly, ofPathMustExist, ofFileMustExist, ofEnableSizing];
          if Execute then
            try
              LoadMappingTable(FileName);
            except
              MessageDlg('Unable to load Mapping Table File', mtError, [mbOk], 0);
              MyMappingTable.setFileName('');
            end;
        except
          MessageDlg('No Mapping Table File Loaded', mtError, [mbOk], 0);
        end;
    except
      MessageDlg('No Mapping Table File Loaded', mtError, [mbOk], 0);
    end;
end;

procedure TfrmMain.acSaveAsExecute(Sender: TObject);
begin
  with TSaveDialog.Create(Self) do
    try
      Filter := 'Mapping Table Files|*.xml';
      DefaultExt := '.xml';
      Title := 'Save Mapping Table File As';
      Options := [ofOverwritePrompt, ofHideReadOnly, ofEnableSizing];
      if Execute then
        begin
          MyMappingTable.setFileName(FileName);
          acSave.Execute;
        end;
    finally
      Free;
    end;
end;

procedure TfrmMain.acSaveExecute(Sender: TObject);
begin
  MyMappingTable.setName(edtName.Text);
  MyMappingTable.setSourceID(edtSourceID.Text);
  MyMappingTable.setSourceApplication(edtSourceApplication.Text);
  MyMappingTable.setSourceVersion(edtSourceVersion.Text);
  MyMappingTable.setHL7SendingApplication(edtHL7SendingApplication.Text);

  MyMappingTable.setActive(cbxActive.Checked);
  MyMappingTable.setSourceTrusted(chkbxSourceTrusted.Checked);

  MyMappingTable.setComment(memComment.Text);

  MyMappingTable.Save;
end;

procedure TfrmMain.acMgrUpdate(Action: TBasicAction; var Handled: Boolean);
begin
  acOpen.Enabled := True;
  acSave.Enabled := (MyMappingTable.getIsOpen);
  acEditUndo.Enabled := (MyMappingTable.getIsOpen);
  acSaveAs.Enabled := (MyMappingTable.getIsOpen);
  acClose.Enabled := (MyMappingTable.getIsOpen);
  acExit.Enabled := True;
  acNewKey.Enabled := (MyMappingTable.getIsOpen);
  acDeleteKey.Enabled := (lvKeys.SelCount > 0);
  acRename.Enabled := (lvKeys.SelCount = 1);
  acValidate.Enabled := (MyMappingTable.getIsOpen);
  acBatchUpdate.Enabled := (MyMappingTable.getIsOpen);
  acInstall.Enabled := (MyMappingTable.getIsOpen) and fInstallable;
  acNewTable.Enabled := True;

  lblMappingTable.Enabled := (MyMappingTable.getIsOpen);
  lblEntries.Enabled := (MyMappingTable.getIsOpen);
  lblMappingLogic.Enabled := (MyMappingTable.getIsOpen);

  lvKeys.Enabled := (MyMappingTable.getIsOpen);

  lblName.Enabled := (MyMappingTable.getIsOpen);
  lblSendingApplication.Enabled := (MyMappingTable.getIsOpen);
  lblSourceID.Enabled := (MyMappingTable.getIsOpen);
  lblSourceApplication.Enabled := (MyMappingTable.getIsOpen);
  lblSourceVersion.Enabled := (MyMappingTable.getIsOpen);
  lblComment.Enabled := (MyMappingTable.getIsOpen);

  edtName.Enabled := (MyMappingTable.getIsOpen);
  edtSourceID.Enabled := (MyMappingTable.getIsOpen);
  edtSourceApplication.Enabled := (MyMappingTable.getIsOpen);
  edtSourceVersion.Enabled := (MyMappingTable.getIsOpen);
  edtHL7SendingApplication.Enabled := (MyMappingTable.getIsOpen);

  memComment.Enabled := (MyMappingTable.getIsOpen);

  chkbxDoNotProcessThisKey.Enabled := (MyMappingTable.getIsOpen) and (lvKeys.SelCount = 1);

  cbxObservation.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  cbxUnit.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  cbxMethod.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  cbxLocation.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  cbxQuality.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  cbxProduct.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);

  lblObservation.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  lblUnit.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  lblMethod.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  lblLocation.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  lblQuality.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);
  lblProduct.Enabled := (MyMappingTable.getIsOpen) and (not chkbxDoNotProcessThisKey.Checked) and (lvKeys.SelCount = 1);

  cbxActive.Enabled := (MyMappingTable.getIsOpen);
  chkbxSourceTrusted.Enabled := (MyMappingTable.getIsOpen);

  Handled := True;
end;

end.

