unit iCPTerminology;
{
   This is the only file to place in the uses clause for Delphi Applications
   that wish to utilize the Clinical Procedures Terminology service.
}
interface

uses
  ComCtrls,
  XMLIntf,
  Classes,
  SysUtils,
  Windows;

type
  ETerminologyLoadError = Exception;
  ETerminologySaveError = Exception;
  ETerminologyDictionaryError = Exception;

  TOnStatusUpdate = procedure(aMsg: string = 'Ready') of object;

  TApplicationItemType = (itTerm, itUnit);
  TTermType = (ttUnknown, ttObservation, ttUnit, ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor, ttTask, ttSchedule, ttDataValue, ttConcept);
  TTermComponentType = (ctUnknown, ctQualifier, ctUnit, ctConversion, ctChildTerm, ctError, ctToDoItem);
  TDataType = (dtNone, dtComplex, dtNumeric, dtPicklist, dtBoolean, dtString, dtRange);
  TScheduleType = (stContinuous, stPRN, stStat, stOneTime);
  TValueType = (vtUnspecified, vtTemperature, vtLength, vtVolume, vtRate, vtTime, vtMass, vtScale);
  TChildValueType = (vtExtract, vtDelimited, vtCalculation);
  TTermToDoStatus = (tdPending, tdClosed, tdDeferred);
  TTermErrorType = (etUnknown, etDataValue, etLogic, etInactiveTerminology, etComponentSetting);
  TTermAttribute = (taID, taName, taAbbreviation, taDisplayName, taTermType, taDataType, taActive, taDescription, taHelpText, taBooleanTrue, taBooleanFalse, taMultiSelect, taVUID);

type
  ITerm = interface;
  ITermComponent = interface;
  ITermToDoItem = interface;
  ITerminologyController = interface;
  ITerminologyEnumerator = interface;
  ITerminologyServer = interface;
  ITerminologyServerList = interface;
  ITermComponentEnumerator = interface;
  ITermError = interface;

  ITermMappingTable = interface;
  ITermMappingTableEntry = interface;
  ITermMappingTableEnumerator = interface;

  ITermComponent = interface(IInterface)
    ['{54575438-F63C-4A30-A03E-824544F55444}']
    function getParentTerm: ITerm; stdcall;
    function getComponentTerm: ITerm; stdcall;
    function getComponentType: TTermComponentType; stdcall;
    function getComponentOrder: integer; stdcall;
    function getComponentDescription: string; stdcall;
    function getActive: boolean; stdcall;
    function getDetailLine: string; stdcall;
    function getComponentDisplayName: string; stdcall;

    procedure setComponentOrder(const aValue: integer); stdcall;
    procedure setComponentDescription(const aValue: string); stdcall;

    procedure Save(aXMLNode: IXMLNode); stdcall;
    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Validate; stdcall;

    function AddToList(aList: TStrings; aUseDisplayName: boolean = False): integer; stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
  end;

  ITermComponentEnumerator = interface(IInterface)
    ['{3D98B9C6-D2AB-4C42-AF87-6B239FA53905}']
    function GetCurrentIndex: integer; stdcall;
    function GetPercentageDone: integer; stdcall;
    function GetCurrentComponent: ITermComponent; stdcall;
    function MoveNext: boolean; stdcall;
  end;

  ITerminologyController = interface(IInterface)
    ['{15C4AC51-0877-4311-9BB6-2E37F537E703}']
    function getDescription: string; stdcall;
    function getDateTimeCreated: TDateTime; stdcall;
    function getDateTimeModified: TDateTime; stdcall;

    procedure setDescription(const aValue: string); stdcall;
    procedure setOnStatusUpdate(aProcedure: TOnStatusUpdate); stdcall;

    function getCount: integer; stdcall;
    function getCountByType(aTermType: TTermType): integer; stdcall;

    function getTerm(aID: string): ITerm; stdcall;
    function getTermName(aID: string): string; stdcall;

    function getModified: boolean; stdcall;
    function getFileName: string; stdcall;

    function getEnumerator: ITerminologyEnumerator; stdcall;

    function getNewTerm(aTermType: TTermType; aDataType: TDataType = dtNone): ITerm; stdcall;
    procedure LoadList(aList: TStrings); overload; stdcall;
    procedure LoadList(aList: TStrings; aTermType: TTermType); overload; stdcall;
    function LoadTermAttributes(aList: TStrings): integer; stdcall;
    function LoadTermTypes(aList: TStrings): integer; stdcall;
    function LoadDataTypes(aList: TStrings): integer; stdcall;
    function LoadToDoStatuses(aList: TStrings): integer; stdcall;
    function IsOpen: boolean; stdcall;
    procedure Load(aFileName: string); stdcall;
    procedure Close; stdcall;
    procedure Save; stdcall;
    procedure SaveAs(aFileName: string); stdcall;
  end;

  ITerminologyEnumerator = interface
    ['{21559E45-596F-4A2B-A3B3-2870D830B45D}']
    function GetCurrentIndex: integer; stdcall;
    function GetPercentageDone: integer; stdcall;
    function GetCurrentTerm: ITerm; stdcall;
    function MoveNext: boolean; stdcall;
  end;

  ITerm = interface(IInterface)
    ['{E02FFEAB-B72B-E77F-31FF-6E074E04164D}']
    function getXMLNode: IXMLNode; stdcall;

    function getAbbreviation: string; stdcall;
    function getActive: boolean; stdcall;
    function getBooleanValueFalse: string; stdcall;
    function getBooleanValueTrue: string; stdcall;
    function getChangeMade: boolean; stdcall;
    function getDataType: TDataType; stdcall;
    function getDateTimeCreated: TDateTime; stdcall;
    function getDescription: string; stdcall;
    function getDisplayName: string; stdcall;
    function getHelpText: string; stdcall;
    function getID: string; stdcall;
    function getMultiSelectPicklist: boolean; stdcall;
    function getSchedule: integer; stdcall;
    function getScheduleType: TScheduleType; stdcall;
    function getSibling(aID: string): ITerm; stdcall;
    function getTerm: string; stdcall;
    function getTermType: TTermType; stdcall;
    function getVUID: string; stdcall;
    function getListIndex(aList: TStrings): integer; stdcall;
    function getComponentCount(aComponentType: TTermComponentType): integer; stdcall;
    function getComponentEnumerator(aComponentType: TTermComponentType): ITermComponentEnumerator; stdcall;
    function getIsValidDefinition: boolean; stdcall;

    procedure setAbbreviation(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setBooleanValueFalse(const aValue: string); stdcall;
    procedure setBooleanValueTrue(const aValue: string); stdcall;
    procedure setDataType(const aValue: TDataType); stdcall;
    procedure setDescription(const aValue: string); stdcall;
    procedure setDisplayName(const aValue: string); stdcall;
    procedure setHelpText(const aValue: string); stdcall;
    procedure setId(const aValue: string); stdcall;
    procedure setMultiSelectPicklist(const aValue: boolean); stdcall;
    procedure setSchedule(const aValue: integer); stdcall;
    procedure setScheduleType(const aValue: TScheduleType); stdcall;
    procedure setTerm(const aValue: string); stdcall;
    procedure setTermType(const aValue: TTermType); stdcall;
    procedure setVuid(const aValue: string); stdcall;
    procedure setChangeMade(const aValue: boolean); stdcall;

    function AddToList(aList: TStrings): integer; stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode; aIncludeAbbv: boolean = False): TTreeNode; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;

    procedure LoadComponents(aList: TStrings; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponents(aList: TStrings; aComponentType: TTermComponentType; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponents(aList: TStrings; aQualifierType: TTermType; aUseDisplayName: boolean = False); overload; stdcall;
    procedure LoadComponentTerms(aList: TStrings; aTermType: TTermType); stdcall;
    procedure LoadErrors(aList: TStrings); stdcall;

    procedure Validate; stdcall;
    function NewComponent(aXMLNode: IXMLNode): ITermComponent; overload;
    function NewComponent(aComponentType: TTermComponentType; aComponentTerm: ITerm): ITermComponent; overload;
    procedure RemoveComponent(aComponent: ITermComponent); stdcall;
    procedure ClearComponents; stdcall;
    function ContainsTerm(aTerm: ITerm): boolean; stdcall;

    function NewError(aType: TTermErrorType; aMessage: string): ITermError; stdcall;
    function NewToDoItem: ITermToDoItem; stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;
  end;

  ITermQualifierPair = interface(ITermComponent)
    ['{88D64F4F-241F-2C38-8BCB-EC9450D49735}']

    function getRanking: double; stdcall;
    function getPairType: TTermType; stdcall;

    procedure setRanking(const aValue: double); stdcall;

    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Save(aXMLNode: IXMLNode); stdcall;
  end;

  ITermUnitConversion = interface(ITermComponent)
    ['{18F60484-0435-6A89-6B9A-5E466F38FFFF}']

    function getConversionFactor: double; stdcall;
    function getConversionOffsetPost: double; stdcall;
    function getConversionOffsetPre: double; stdcall;
    function getDecimalPrecision: integer; stdcall;
    function getUnitsAreEqual: boolean; stdcall;

    procedure setConversionFactor(const aValue: double); stdcall;
    procedure setConversionOffsetPost(const aValue: double); stdcall;
    procedure setConversionOffsetPre(const aValue: double); stdcall;
    procedure setDecimalPrecision(const aValue: integer); stdcall;
    procedure setUnitsAreEqual(const aValue: boolean); stdcall;

    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Save(aXMLNode: IXMLNode); stdcall;
  end;

  ITermUnitPair = interface(ITermComponent)
    ['{F02DE6C9-0767-495F-B01D-746BF962594D}']
    function getDecPrecision: integer; stdcall;
    function getMaxValue: double; stdcall;
    function getMinValue: double; stdcall;
    function getReferenceHigh: double; stdcall;
    function getReferenceLow: double; stdcall;

    procedure setDecPrecision(const aValue: integer); stdcall;
    procedure setMaxValue(const aValue: double); stdcall;
    procedure setMinValue(const aValue: double); stdcall;
    procedure setReferenceHigh(const aValue: double); stdcall;
    procedure setReferenceLow(const aValue: double); stdcall;

    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Save(aXMLNode: IXMLNode); stdcall;
  end;

  ITermChildPair = interface(ITermComponent)
    ['{08709060-62C0-F3C1-5A95-3C05ADFF664D}']
    function getValueType: TChildValueType; stdcall;
    function getValueDelimiter: Char; stdcall;
    function getValueStartPosition: integer; stdcall;
    function getValueStopPosition: integer; stdcall;
    function getValueUnit: ITerm; stdcall;
    function getValueUnitID: string; stdcall;

    procedure setValueType(const aValue: TChildValueType); stdcall;
    procedure setValueDelimiter(const aValue: Char); stdcall;
    procedure setValueStartPosition(const aValue: integer); stdcall;
    procedure setValueStopPosition(const aValue: integer); stdcall;
    procedure setValueUnit(const aValue: ITerm); stdcall;

    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Save(aXMLNode: IXMLNode); stdcall;
  end;

  ITermToDoItem = interface(ITermComponent)
    ['{0DF7C778-1A5F-4A22-A156-94CBD0DE844D}']
    function getID: string; stdcall;
    function getItemText: string; stdcall;
    function getStatus: TTermToDoStatus; stdcall;
    function getEnteredBy: string; stdcall;
    function getEnteredAtWorkstation: string; stdcall;
    function getDateTimeEntered: TDateTime; stdcall;
    function getDateTimeCompleted: TDateTime; stdcall;

    procedure setItemText(const aValue: string); stdcall;
    procedure setStatus(const aValue: TTermToDoStatus); stdcall;
    procedure setDateTimeCompleted(const aValue: TDateTime); stdcall;
  end;

  ITerminologyServer = interface(IInterface)
    ['{6C86C548-639E-4495-B6B1-6FB546FBDEDF}']
    function getID: string; stdcall;
    function getServerName: string; stdcall;
    function getServerIP: string; stdcall;
    function getServerPort: string; stdcall;
    function getAccessCode: string; stdcall;
    function getVerifyCode: string; stdcall;
    function getHint: string; stdcall;

    procedure setID(const aValue: string); stdcall;
    procedure setServerName(const aValue: string); stdcall;
    procedure setServerIP(const aValue: string); stdcall;
    procedure setServerPort(const aValue: string); stdcall;
    procedure setAccessCode(const aValue: string); stdcall;
    procedure setVerifyCode(const aValue: string); stdcall;
    procedure setHint(const aValue: string); stdcall;

    procedure Delete; stdcall;

    function AddToList(aList: TStrings): boolean; stdcall;
    function AddToListView(aListView: TListView): TListItem; stdcall;
  end;

  ITerminologyServerList = interface(IInterface)
    ['{5AC544D9-2087-4A93-B3E8-9C36B1105839}']
    function getCount: integer; stdcall;
    function getFirstItem: ITerminologyServer; stdcall;
    function getNextItem(aServer: ITerminologyServer): ITerminologyServer; stdcall;
    function getItem(aID: string): ITerminologyServer; stdcall;
    function getXMLNode: IXMLNode; stdcall;

    function NewTerminologyServer: ITerminologyServer; stdcall;
    procedure AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode); stdcall;
    procedure LoadListView(aListView: TListView); stdcall;
    procedure LoadList(aList: TStrings); stdcall;
    procedure Load; stdcall;
    procedure Delete(aItem: ITerminologyServer); stdcall;
  end;

  ITermError = interface(ITermComponent)
    ['{0FA91F03-A30B-4A70-BF80-A3B2C4660D9B}']
    function getErrorType: TTermErrorType; stdcall;
    function getErrorMessage: string; stdcall;
    function getDateTimeFound: TDateTime; stdcall;
    function getDetailLine: string; stdcall;

    procedure setErrorType(const aValue: TTermErrorType); stdcall;
    procedure setErrorMessage(const aValue: string); stdcall;
    procedure setDateTimeFound(const aValue: TDateTime); stdcall;

    procedure Load(aXMLNode: IXMLNode); stdcall;
    procedure Save(aXMLNode: IXMLNode); stdcall;
  end;

  IHTMLGenerator = interface
    ['{0FC0A2BA-EB51-4F4C-AC34-127B12FC5CC1}']
    procedure GenerateHTML(aStrings: TStrings); stdcall;
  end;

  ITermMappingTable = interface
    ['{E7E450C3-CBEB-4EC4-B78C-22BEC28D23C0}']
    function getCreatedBy: string; stdcall;
    function getCreatedDateTime: TDateTime; stdcall;
    function getCreatedOnWkstn: string; stdcall;
    function getUpdatedBy: string; stdcall;
    function getUpdatedDateTime: TDateTime; stdcall;
    function getUpdatedOnWkstn: string; stdcall;
    function getID: string; stdcall;
    function getName: string; stdcall;
    function getActive: boolean; stdcall;
    function getComment: string; stdcall;
    function getSourceID: string; stdcall;
    function getSourceApplication: string; stdcall;
    function getSourceVersion: string; stdcall;
    function getSourceTrusted: boolean; stdcall;
    function getHL7SendingApplication: string; stdcall;
    function getIsOpen: boolean; stdcall;
    function getFileName: string; stdcall;
    function getChangeMade: boolean; stdcall;

    procedure setUpdatedBy(const aValue: string); stdcall;
    procedure setUpdatedDateTime(const aValue: TDateTime); stdcall;
    procedure setUpdatedOnWkstn(const aValue: string); stdcall;
    procedure setName(const aValue: string); stdcall;
    procedure setActive(const aValue: boolean); stdcall;
    procedure setComment(const aValue: string); stdcall;
    procedure setSourceID(const aValue: string); stdcall;
    procedure setSourceApplication(const aValue: string); stdcall;
    procedure setSourceVersion(const aValue: string); stdcall;
    procedure setSourceTrusted(const aValue: boolean); stdcall;
    procedure setHL7SendingApplication(const aValue: string); stdcall;
    procedure setOnStatusUpdate(aProcedure: TOnStatusUpdate); stdcall;
    procedure setFileName(const aValue: string); stdcall;
    procedure setChangeMade(const aValue: boolean); stdcall;

    function NewEntry(aKey: string): ITermMappingTableEntry;
    procedure DeleteEntry(aTermMappingTableEntry: ITermMappingTableEntry);
    function NewEnumerator: ITermMappingTableEnumerator;

    procedure Load(aFileName: string); stdcall;
    procedure Close; stdcall;
    procedure Save; stdcall;
  end;

  ITermMappingTableEntry = interface
    ['{EE45D4FA-61B3-477E-8510-290EBD7631C2}']
    function getMappedTerm(aTermType: TTermType): ITerm; stdcall;
    function getMappedTermID(aTermType: TTermType): string; stdcall;
    function getVendorKey: string; stdcall;
    function getDoNotProcess: boolean; stdcall;

    function getMappingIDs: string; stdcall;
    function getMappingDisplay: string; stdcall;
    function getXMLNode: IXMLNode; stdcall;

    procedure setVendorKey(const aValue: string); stdcall;
    procedure setDoNotProcess(const aValue: boolean); stdcall;
    procedure setMappedTerm(const aTermType: TTermType; const aValue: ITerm); stdcall;

    procedure Load; stdcall;
    procedure Save; stdcall;

    function AddToList(aList: TStrings): integer; overload; stdcall;
    function AddToList(aListView: TListView; aImageIndex: integer = -1): TListItem; overload; stdcall;
  end;

  ITermMappingTableEnumerator = interface
    ['{4867B4C7-135D-411B-82E6-DB3477A5C4E7}']
    function GetCurrentIndex: integer; stdcall;
    function GetPercentageDone: integer; stdcall;
    function GetCurrentEntry: ITermMappingTableEntry; stdcall;
    function MoveNext: boolean; stdcall;
  end;

const
  TERM_ATTRIBUTE: array[TTermAttribute] of string = ('ID', 'Name', 'Abbreviation', 'Display Name', 'Term Type', 'Data Type', 'Active', 'Description', 'Help Text', 'Boolean True', 'Boolean False', 'Multi-Select', 'VUID');

  TERM_TYPE: array[TTermType] of string = ('UNKNOWN', 'OBSERVATION', 'UNIT', 'METHOD', 'POSITION', 'LOCATION', 'QUALITY', 'PRODUCT', 'NULL FLAVOR', 'TASK', 'SCHEDULE', 'DATA VALUE', 'CONCEPT');

  TERM_TYPE_DISPLAY: array[TTermType] of string = ('Unknown', 'Observations', 'Units', 'Methods', 'Positions', 'Locations', 'Qualities', 'Products', 'Null Flavors', 'Tasks', 'Schedules', 'Picklist Values', 'Concepts');

  TERM_TYPE_PAIR: array[TTermType] of string = ('Unknown', 'Child Pair', 'Unit Pair', 'Method Pair', 'Position Pair', 'Location Pair', 'Quality Pair', 'Product Pair', 'Null Flavor Pair', 'Task Pair', 'Schedule Pair', 'Data Value Pair', 'Concept Pair');

  TERM_TYPE_XML: array[TTermType] of string = ('Unknown', 'TERM_ID', 'UNIT_ID', 'METHOD_ID', 'POSITION_ID', 'LOCATION_ID', 'QUALITY_ID', 'PRODUCT_ID', 'NULL_FLAVOR_ID', 'TASK_ID', 'SCHEDULE_ID', 'DATA_VALUE_ID', 'CONCEPT_ID');

  TERM_QUALIFIERS: set of TTermType = [ttMethod, ttPosition, ttLocation, ttQuality, ttProduct, ttNullFlavor, ttDataValue];

  TERM_DATATYPE: array[TDataType] of string = ('None', 'Complex', 'Numeric', 'Picklist', 'Boolean', 'String', 'Range');

  TERM_TODO_STATUS: array[TTermToDoStatus] of string = ('Pending', 'Closed', 'Deferred');

  TERM_ERROR_TYPE: array[TTermErrorType] of string = ('Unknown', 'Data Value', 'Definition Logic', 'Inactive Term In Use', 'Component Setting');

  TERM_COMPONENT_TYPE: array[TTermComponentType] of string = ('Unknown', 'Qualifiers', 'Units of Measure', 'Conversion Methods', 'Child Terms', 'Errors', 'To Do Items');

  BOOLEAN_TO_INT: array[boolean] of integer = (0, 1);

  BOOLEAN_TO_STR: array[boolean] of string = ('False', 'True');

  DO_NOT_PROCESS_ID = '{4615254C-EC67-46D0-BF70-9A54BEB4B32D}';

function NewTerminologyController: ITerminologyController;
function NewMappingTable(aTerminologyController: ITerminologyController): ITermMappingTable;

function GetUserFromWindows: string;
function GetComputerNetName: string;
function GetPercentageDone(aDone, aTotal: integer): string;

implementation

uses
  oTerminologyController,
  oTermMappingTable;

function GetUserFromWindows: string;
var
  UserName: string;
  UserNameLen: Dword;
begin
  UserNameLen := 255;
  SetLength(userName, UserNameLen);
  if GetUserName(PChar(UserName), UserNameLen) then
    Result := Copy(UserName, 1, UserNameLen - 1)
  else
    Result := 'Unknown';
end;

function GetComputerNetName: string;
var
  buffer: array[0..255] of char;
  size: dword;
begin
  size := 256;
  if GetComputerName(buffer, size) then
    Result := buffer
  else
    Result := ''
end;

function GetPercentageDone(aDone, aTotal: integer): string;
begin
  Result := Format('%d%%', [trunc((aDone / aTotal) * 100)]);
end;

function NewTerminologyController: ITerminologyController;
begin
  TTerminologyController.Create.GetInterface(ITerminologyController, Result);
end;

function NewMappingTable(aTerminologyController: ITerminologyController): ITermMappingTable;
begin
  TTermMappingTable.Create(aTerminologyController).getInterface(ITermMappingTable, Result);
end;

(*
function NewTerm(aTerminology: ITerminology; aXMLNode: IXMLNode): ITerm;
{NOTICE: ONLY BUILDS THE BASIC HEADER RECORD OF THE TERM, NOT THE FULL DEFINITION}
begin
  TTerm.Create(aTerminology, aXMLNode, MyUserName, MyComputerName).GetInterface(ITerm, Result);
end;
*)

(*
function NewTermComponent(aParentTerm: ITerm; aComponentType: TTermComponentType; aComponentTerm: ITerm): ITermComponent;
begin
  case aComponentType of
    ctQualifier: TTermQualifierPair.Create(aParentTerm, aComponentTerm).GetInterface(ITermQualifierPair, Result);
    ctUnit: TTermUnitPair.Create(aParentTerm, aComponentTerm).GetInterface(ITermUnitPair, Result);
    ctConversion: TTermUnitConversion.Create(aParentTerm, aComponentTerm).GetInterface(ITermUnitConversion, Result);
    ctChildTerm: TTermChildPair.Create(aParentTerm, aComponentTerm).GetInterface(ITermChildPair, Result);
    ctError: TTermError.Create(aParentTerm).GetInterface(ITermError, Result);
    ctToDoItem: TTermToDoItem.Create(aParentTerm).GetInterface(ITermToDoItem, Result);
  else
    Result := nil;
  end;
end;

function NewTermComponent(aParentTerm: ITerm; aXMLNode: IXMLNode): ITermComponent;
var
  aType: TTermComponentType;
  aComp: ITerm;
begin
  try
    aType := TTermComponentType(XMLValueAsInt(aXMLNode, 'COMPONENT_TYPE'));
    aComp := aParentTerm.getSibling(XMLValueAsStr(aXMLNode, 'COMPONENT_TERM'));

    with NewTermComponent(aParentTerm, aType, aComp) do
      begin
        Load(aXMLNode);
        QueryInterface(ITermComponent, Result);
      end;
  except
    Result := nil;
  end;
end;
*)

end.

