unit oTermQualifierPair;

interface

uses
  ComCtrls,
  XMLIntf,
  SysUtils,
  Classes,
  uXML,
  oTermComponent,
  iCPTerminology;

type
  TTermQualifierPair = class(TTermComponent, ITermComponent, ITermQualifierPair)
  private
    fRanking: double;

  public
    constructor Create(aParentTerm: ITerm; aQualifier: ITerm);
    destructor Destroy; override;

    function getRanking: double; stdcall;
    function getPairType: TTermType; stdcall;
    function getComponentDisplayName: string; override; stdcall;

    procedure setRanking(const aValue: double); stdcall;

    procedure Load(aXMLNode: IXMLNode); override; stdcall;
    procedure Save(aXMLNode: IXMLNode); override; stdcall;
    procedure Validate; override; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_RANKING = 'RANKING';

  {Default values for the properties}
  DEFAULT_RANKING = 0.0;

constructor TTermQualifierPair.Create(aParentTerm: ITerm; aQualifier: ITerm);
begin
  inherited Create(ctQualifier, aParentTerm, aQualifier);
  fRanking := DEFAULT_RANKING;
end;

destructor TTermQualifierPair.Destroy;
begin
  inherited;
end;

procedure TTermQualifierPair.Load(aXMLNode: IXMLNode);
begin
  try
    inherited Load(aXMLNode);
    fRanking := XMLValueAsDouble(aXMLNode, XML_RANKING, DEFAULT_RANKING);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermQualifierPair.Save(aXMLNode: IXMLNode);
begin
  try
    inherited Save(aXMLNode);
    aXMLNode.ChildValues[XML_RANKING] := fRanking;
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to save %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

{ Private getters }

function TTermQualifierPair.getComponentDisplayName: string;
var
  aFmt: string;
begin
  if Frac(fRanking) > 0 then
    aFmt := Format('%%s (%%0.%df)', [Length(FloatToStr(Frac(fRanking))) - 2])
  else
    aFmt := '%s (%0.0f)';
  Result := Format(aFmt, [getComponentTerm.getTerm, fRanking]);
end;

function TTermQualifierPair.getPairType: TTermType;
begin
  Result := getComponentTerm.getTermType;
end;

function TTermQualifierPair.getRanking: double;
begin
  Result := fRanking;
end;

{ Private Setters }

procedure TTermQualifierPair.setRanking(const aValue: double);
begin
  fRanking := aValue;
end;

procedure TTermQualifierPair.Validate;
begin
  inherited;
  if getComponentTerm = nil then
    getParentTerm.NewError(etComponentSetting, 'Missing Component Term')
  else
    if not getComponentTerm.getActive then
      getParentTerm.NewError(etComponentSetting, Format('Inactive Qualifier %s assigned', [getComponentTerm.getTerm]));
end;

end.

