unit uAddEdtServer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uServers, ComCtrls, StdCtrls, ExtCtrls, Mask;

type
  TfrmAddEditServers = class(TForm)
    lvServers: TListView;
    btnNewServer: TButton;
    btnDeleteServer: TButton;
    Panel1: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    Panel2: TPanel;
    Label1: TLabel;
    edtServerName: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    meServerIP: TMaskEdit;
    meServerPort: TMaskEdit;
    procedure lvServersMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnNewServerClick(Sender: TObject);
    procedure btnDeleteServerClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure edtServerNameChange(Sender: TObject);
    procedure meServerIPChange(Sender: TObject);
    procedure meServerPortChange(Sender: TObject);
  private
    { Private declarations }
    FServers: TServerList;
    procedure RedrawServers;
    procedure EnableControls;
    procedure DisableControls;
  public
    { Public declarations }
    property Servers: TServerList read FServers write FServers;
  end;

var
  frmAddEditServers: TfrmAddEditServers;

implementation

var
  CurrentServer: TServer;

{
  Yes, We're doing an AWFUL lot of repainting of the server dialog when someone
  decides to edit the text.  We may want to try to find a better way to do this
  later.
}

{$R *.dfm}

procedure TfrmAddEditServers.btnDeleteServerClick(Sender: TObject);
begin
  if MessageDlg('Are you sure you wish to delete this server?', mtConfirmation,
    [mbYes,mbNo],0) = mrYes then
  begin
    FServers.Delete(lvServers.Selected.Index);
    lvServers.Items.Delete(lvServers.Selected.Index);
  end;
end;

procedure TfrmAddEditServers.btnNewServerClick(Sender: TObject);
begin
  CurrentServer := FServers.Add;
  CurrentServer.ServerName:= 'Localhost';
  CurrentServer.ServerIP := '127.0.0.1';
  CurrentServer.ServerPort := 1234;
  
  lvServers.Items.Add.Data := CurrentServer;
  lvServers.Selected := lvServers.Items[lvServers.Items.Count - 1];
  RedrawServers;
  edtServerName.SetFocus;
end;

procedure TfrmAddEditServers.DisableControls;
begin
  edtServerName.Text := '';
  meServerIP.Text := '';
  meServerPort.Text := '0';
  edtServerName.Enabled := False;
  meServerIP.Enabled := False;
  meServerPort.Enabled := False;
  btnDeleteServer.Enabled := False;
end;

procedure TfrmAddEditServers.edtServerNameChange(Sender: TObject);
begin
  if Assigned(CurrentServer) then
  begin
    CurrentServer.ServerName := edtServerName.Text;
    RedrawServers;
  end;
end;

procedure TfrmAddEditServers.EnableControls;
begin
  edtServerName.Enabled := True;
  meServerIP.Enabled := True;
  meServerPort.Enabled := True;

  CurrentServer := TServer(lvServers.Selected.Data);

  edtServerName.Text := CurrentServer.ServerName;
  meServerIP.Text := CurrentServer.ServerIP;
  meServerPort.Text := IntToStr(CurrentServer.ServerPort);
end;

procedure TfrmAddEditServers.FormShow(Sender: TObject);
begin
  CurrentServer := nil;
  RedrawServers;
end;

procedure TfrmAddEditServers.lvServersMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(lvServers.Selected) then
  begin
    EnableControls;
  end
  else
  begin
  end;
end;

procedure TfrmAddEditServers.meServerIPChange(Sender: TObject);
begin
  if Assigned(CurrentServer) then
  begin
    CurrentServer.ServerIP := meServerIP.Text;
    RedrawServers;
  end;
end;

procedure TfrmAddEditServers.meServerPortChange(Sender: TObject);
begin
  if Assigned(CurrentServer) then
  begin
    CurrentServer.ServerPort := StrToIntDef(Trim(meServerPort.Text),0);
    RedrawServers;
  end;
end;

procedure TfrmAddEditServers.RedrawServers;
var
  Enum: TCollectionEnumerator;
  Server: TServer;
begin
  lvServers.Clear;
  Enum := FServers.GetEnumerator;
  if Assigned(Enum) then
  begin
    lvServers.Items.BeginUpdate;
    while Enum.MoveNext do
    begin
      Server := TServer(Enum.GetCurrent);
      with lvServers.Items.Add do
      begin
        Caption := Server.ServerName;
        SubItems.Add(Server.ServerIP);
        SubItems.Add(IntToStr(Server.ServerPort));
        Data := Server;
      end;
    end;
    lvServers.Items.EndUpdate;
  end;
end;

end.
