unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, IdBaseComponent, IdHL7, IdTCPConnection,
  uServerList, uServers;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    btnSend: TButton;
    Bevel1: TBevel;
    Panel2: TPanel;
    StatusBar1: TStatusBar;
    Label3: TLabel;
    mmoAck: TMemo;
    Panel3: TPanel;
    Label4: TLabel;
    mmoMessage: TMemo;
    hl7ComLink: TIdHL7;
    Label5: TLabel;
    cbConnectionList: TComboBox;
    btnNewConnection: TButton;
    lblIPAddress: TLabel;
    lblPort: TLabel;
    btnLoadMsg: TButton;
    odFindMessage: TOpenDialog;
    procedure btnSendClick(Sender: TObject);
    procedure btnLoadMsgClick(Sender: TObject);
    procedure hl7ComLinkDisconnect(Sender: TObject);
    procedure hl7ComLinkConnect(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnNewConnectionClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbConnectionListSelect(Sender: TObject);
  private
    { Private declarations }
    Servers: TServers;
    procedure ReloadConnectionList;
  public
    { Public declarations }
    procedure HL7MessageArrive(ASender: TObject; AConnection: TIdTCPConnection; AMsg: String);
    procedure HL7MessageReceive(ASender: TObject; AConnection: TIdTCPConnection; AMsg: String;
      var VHandled: Boolean; var VReply: string);
  end;

var
  frmMain: TfrmMain;

implementation

const
  WINDOW_TITLE = 'Simple HL7 Generator';
  SERVERS_FILE = 'SERVERS.XML';
  ID = '${ID}';
  DATE = '${DATE}';
  DATETIME = '${DATETIME}';

function GenMsgID: string;
begin
  result := FormatDateTime('yyyymmddhhnnsszzz', Now);
end;

{$R *.dfm}

procedure TfrmMain.btnLoadMsgClick(Sender: TObject);
begin
  if odFindMessage.Execute then
  begin
    mmoMessage.Lines.LoadFromFile(odFindMessage.FileName);
  end;
end;

procedure TfrmMain.btnNewConnectionClick(Sender: TObject);
var
  ServersDlg: TfrmServers;
begin
  ServersDlg := TfrmServers.Create(nil);
  try
    ServersDlg.Servers := Servers;
    if ServersDlg.ShowModal = mrOK then
      if MessageDlg('Do you wish to save changes to the server list?', mtConfirmation, [mbOK, mbCancel],0) = mrOK then
        Servers.Save(SERVERS_FILE);
    ReloadConnectionList;
  finally
    FreeAndNil(ServersDlg);
  end;
end;

procedure TfrmMain.btnSendClick(Sender: TObject);
var
  Reply: string;
  HL7Message: string;
  I: Integer;
  LResult: TSendResponse;
begin
  if hl7ComLink.Status in [isStopped, isNotConnected] then
  begin
    hl7ComLink.Address := lblIPAddress.Caption;
    hl7ComLink.Port := StrToIntDef(lblPort.Caption,0);
    hl7ComLink.Start;
    //hl7ComLink.WaitForConnection(1000);
  end;
  
  HL7Message := '';
  for I := 0 to mmoMessage.Lines.Count - 1 do
    HL7Message := HL7Message + mmoMessage.Lines[I] + #13;

  HL7Message := StringReplace(HL7Message, ID, GenMsgID, [rfReplaceAll]);
  HL7Message := StringReplace(HL7Message, DATE, FormatDateTime('yyyymmdd', now), [rfReplaceAll]);
  HL7Message := StringReplace(HL7Message, DATETIME, FormatDateTime('yyyymmddhhnnss', now), [rfReplaceAll]);

  hl7ComLink.SendMessage(HL7Message);
  repeat
      sleep(20);
      LResult := hl7ComLink.GetReply(Reply);
  until LResult <> srNone;

  case LResult of
  srNoConnection:
    begin
      // you tried to send but there was no connection
      StatusBar1.SimpleText := 'Tried to send message but no connection.  Time: ' + FormatDateTime('hh:nn:ss', now);
    end;
  srSent:
    begin
      // you asked to send without waiting, and it has been done
      StatusBar1.SimpleText := 'Message sent.  Response not received.  Time: ' + FormatDateTime('hh:nn:ss', now);
    end;
  srOK:
    begin
      // sent ok, and response returned
      StatusBar1.SimpleText := 'Message sent.  Response received.  Time: ' + FormatDateTime('hh:nn:ss', now);
    end;
  srTimeout:
    begin

    end;
  end;
  mmoAck.Lines.Text := Reply;
end;

procedure TfrmMain.cbConnectionListSelect(Sender: TObject);
var
  Item: TServerItem;
begin
  Item := TServerItem(cbConnectionList.Items.Objects[cbConnectionList.ItemIndex]);
  lblIPAddress.Caption := Item.IP;
  lblPort.Caption := IntToStr(Item.Port);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Servers := TServers.Create;
  if not FileExists(SERVERS_FILE) then
    Servers.Save(SERVERS_FILE);
  Servers.Load(SERVERS_FILE);
  ReloadConnectionList;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if hl7ComLink.Connected then
    hl7ComLink.Stop;
  FreeAndNil(Servers);
end;

procedure TfrmMain.hl7ComLinkConnect(Sender: TObject);
begin
  frmMain.Caption := WINDOW_TITLE + ' Connected';
end;

procedure TfrmMain.hl7ComLinkDisconnect(Sender: TObject);
begin
  frmMain.Caption := WINDOW_TITLE + ' - Disconnected';
end;

procedure TfrmMain.HL7MessageArrive(ASender: TObject;
  AConnection: TIdTCPConnection; AMsg: String);
begin
  mmoAck.Lines.Text := AMsg;
end;

procedure TfrmMain.HL7MessageReceive(ASender: TObject;
  AConnection: TIdTCPConnection; AMsg: String; var VHandled: Boolean;
  var VReply: string);
begin
  mmoAck.Lines.Text := AMsg;
  StatusBar1.SimpleText := 'Message received.  Time: ' + FormatDateTime('hh:nn:ss', now);
  VHandled := true;
end;

procedure TfrmMain.ReloadConnectionList;
var
  Enum: TCollectionEnumerator;
  Item: TServerItem;
begin
  cbConnectionList.Clear;
  Enum := Servers.GetEnumerator;
  while Enum.MoveNext do
  begin
    Item := TServerItem(Enum.Current);
    cbConnectionList.AddItem(Item.Name, Item);
  end;
  if cbConnectionList.Items.Count > 0 then
  begin
    cbConnectionList.ItemIndex := 0;
    cbConnectionListSelect(nil);
  end;
end;

end.
